@@ -11,10 +11,14 @@ EXCEL = {
     },
     "cells": {
         # like 7
-        "constant": {
+        "constant-numeric": {
             "prefix": "",
             "suffix": "",
         },
+        "constant-string": {
+            "prefix": '"',
+            "suffix": '"',
+        },
         "variable": {
             "prefix": "",
             "suffix": "",
@@ -177,6 +181,10 @@ EXCEL = {
             "prefix": "IRR(",
             "suffix": ")",
         },
+        "match-negative-before-positive": {
+            "prefix": "MATCH(0,",
+            "suffix": ")",
+        },
         # BASIC OPERATIONS
         "exponential": {
             "prefix": "EXP(",
@@ -291,10 +299,14 @@ NATIVE = {
     },
     "cells": {
         # like 7
-        "constant": {
+        "constant-numeric": {
             "prefix": "",
             "suffix": "",
         },
+        "constant-string": {
+            "prefix": '"',
+            "suffix": '"',
+        },
         "variable": {
             "prefix": "value of variable '",
             "suffix": "'",
@@ -450,11 +462,16 @@ NATIVE = {
             "suffix": "",
         },
         "count": {
-            "prefix": " number of items in the slice ",
+            "prefix": "number of items in the slice ",
             "suffix": "",
         },
         "irr": {
-            "prefix": " Internal Rate of Return of ",
+            "prefix": "Internal Rate of Return of ",
+            "suffix": "",
+        },
+        "match-negative-before-positive": {
+            "prefix": "position of the step where numbers "
+                      "become to be positive of ",
             "suffix": "",
         },
         # BASIC OPERATIONS
@@ -571,10 +588,14 @@ PYTHON_NUMPY = {
     },
     "cells": {
         # like 7
-        "constant": {
+        "constant-numeric": {
             "prefix": "",
             "suffix": "",
         },
+        "constant-string": {
+            "prefix": '"',
+            "suffix": '"',
+        },
         "empty": {
             "content": ""
         },
@@ -737,6 +758,10 @@ PYTHON_NUMPY = {
             "prefix": "npf.irr(",
             "suffix": ")",
         },
+        "match-negative-before-positive": {
+            "prefix": "np.argmin(",
+            "suffix": "<0)",
+        },
         # BASIC OPERATIONS
         "exponential": {
             "prefix": "np.exp(",
@@ -872,8 +897,13 @@ GRAMMAR_PATTERN: dict = {
     "cells": {
         # Define the basic words related to a cell
 
-        # Constant definition (like 7):
-        "constant": {
+        # Numeric constant definition (like 7):
+        "constant-numeric": {
+            "prefix": str,
+            "suffix": str,
+        },
+        # String constant definition (like "hello"):
+        "constant-string": {
             "prefix": str,
             "suffix": str,
         },
@@ -1080,6 +1110,12 @@ GRAMMAR_PATTERN: dict = {
             "prefix": str,
             "suffix": str,
         },
+        # Match the position of the last negative number before first
+        #   negative number in a sequence (works only for single row/column).
+        "match-negative-before-positive": {
+            "prefix": str,
+            "suffix": str,
+        },
 
         # === BASIC UNARY OPERATIONS (that takes only one operand) ===
         # Exponential (natural) of the argument
