@@ -7,9 +7,10 @@ from numbers import Number
 import xlsxwriter
 import numpy
 
-from .cell import Cell
+from .cell import Cell, CellValueError
 from .cell_type import CellType
 from .cell_indices import CellIndices
+from .skipped_label import SkippedLabel
 
 # ==== TYPES ====
 # Type for the output dictionary with the logic:
@@ -159,7 +160,8 @@ class Serialization(abc.ABC):
                          "value": "Value",
                          "description": "Description"
                      }),
-                 values_only: bool = False
+                 values_only: bool = False,
+                 skipped_label_replacement: str = ''
                  ) -> None:
         """Export the values inside Spreadsheet instance to the
             Excel 2010 compatible .xslx file
@@ -180,6 +182,8 @@ class Serialization(abc.ABC):
                 for the sheet with variables (first row in the sheet).
             values_only (bool): If true, only values (and not formulas) are
                 exported.
+            skipped_label_replacement (str): Replacement for the SkippedLabel
+                instances.
         """
         # Quick sanity check
         if ".xlsx" not in file_path[-5:]:
@@ -203,6 +207,7 @@ class Serialization(abc.ABC):
             for name, value in self._get_variables().variables_dict.items():
                 workbook.define_name(name, str(value['value']))
         else:
+            # Add variable
             variables_sheet = workbook.add_worksheet(name=variables_sheet_name)
             # Insert header (labels)
             variables_sheet.write(0, 0, variables_sheet_header['name'],
@@ -213,9 +218,17 @@ class Serialization(abc.ABC):
                                   col_label_format)
             row_idx = 1
             for var_n, var_v in self._get_variables().variables_dict.items():
+                # Format the variable style
+                try:
+                    style_var = self._get_variables().excel_format[var_n]
+                    variable_style = workbook.add_format(style_var)
+                except KeyError:
+                    variable_style = None
                 # Insert variables to the sheet
                 variables_sheet.write(row_idx, 0, var_n)
-                variables_sheet.write(row_idx, 1, var_v['value'])
+                variables_sheet.write(
+                    row_idx, 1, var_v['value'], variable_style
+                )
                 variables_sheet.write(row_idx, 2, var_v['description'])
                 # Register variable
                 workbook.define_name(
@@ -243,7 +256,14 @@ class Serialization(abc.ABC):
                     else:
                         cell_format = None
                     # Write actual data
-                    if values_only or (cell.cell_type == CellType.value_only):
+                    if type(cell.value) == CellValueError:
+                        worksheet.write_formula(row_idx + offset_row,
+                                                col_idx + offset_col,
+                                                cell.parse['excel'],
+                                                value='#VALUE!',
+                                                cell_format=cell_format)
+                    elif values_only or (
+                            cell.cell_type == CellType.value_only):
                         # If the cell is a value only, use method 'write'
                         worksheet.write(row_idx + offset_row,
                                         col_idx + offset_col,
@@ -260,44 +280,58 @@ class Serialization(abc.ABC):
         if self.cell_indices.excel_append_column_labels:
             # Add labels of column
             for col_idx in range(self.shape[1]):
+                col_lbl = self.cell_indices.columns_labels[
+                                    # Reflect the export offset
+                                    col_idx + self.export_offset[1]
+                                ].replace(' ', spaces_replacement)
+                if isinstance(col_lbl, SkippedLabel):
+                    col_lbl = skipped_label_replacement
                 worksheet.write(0,
                                 col_idx + int(
                                     self.cell_indices.excel_append_row_labels
                                 ),
-                                self.cell_indices.columns_labels[
-                                    # Reflect the export offset
-                                    col_idx + self.export_offset[1]
-                                ].replace(' ', spaces_replacement),
+                                col_lbl,
                                 col_label_format)
         if self.cell_indices.excel_append_row_labels:
             # Add labels for rows
             for row_idx in range(self.shape[0]):
+                # Reflect the export offset
+                row_lbl = self.cell_indices.rows_labels[
+                    row_idx + self.export_offset[0]
+                ].replace(' ', spaces_replacement)
+                if isinstance(row_lbl, SkippedLabel):
+                    row_lbl = skipped_label_replacement
                 worksheet.write(row_idx + int(
                     self.cell_indices.excel_append_column_labels
                 ),
                                 0,
-                                self.cell_indices.rows_labels[
-                                    # Reflect the export offset
-                                    row_idx + self.export_offset[0]
-                                ].replace(' ', spaces_replacement),
+                                row_lbl,
                                 row_label_format)
         # Store results
         workbook.close()
 
     def to_dictionary(self,
                       languages: List[str] = None,
+                      use_language_for_description: Optional[str] = None,
                       /, *,  # noqa E999
                       by_row: bool = True,
                       languages_pseudonyms: List[str] = None,
                       spaces_replacement: str = ' ',
                       skip_nan_cell: bool = False,
                       nan_replacement: object = None,
-                      append_dict: dict = {}) -> T_out_dict:
+                      error_replacement: object = None,
+                      append_dict: dict = {},
+                      generate_schema: bool = False
+                      ) -> T_out_dict:
         """Export this spreadsheet to the dictionary.
 
         Args:
             languages (List[str]): List of languages that should be exported.
                 If it has value None, all the languages are exported.
+            use_language_for_description (Optional[str]): If set-up (using
+                the language name), description field is set to be either
+                the description value (if defined) or the value of this
+                language.
             by_row (bool): If True, rows are the first indices and columns
                 are the second in the order. If False it is vice-versa.
             languages_pseudonyms (List[str]): Rename languages to the strings
@@ -305,8 +339,10 @@ class Serialization(abc.ABC):
             spaces_replacement (str): All the spaces in the rows and columns
                 descriptions (labels) are replaced with this string.
             skip_nan_cell (bool): If True, None (NaN) values are skipped.
-            nan_replacement (object): Replacement for the None (NaN) value
+            nan_replacement (object): Replacement for the None (NaN) value.
+            error_replacement (object): Replacement for the error value.
             append_dict (dict): Append this dictionary to output.
+            generate_schema (bool): If true, returns the JSON schema.
 
         Returns:
             dict:
@@ -315,6 +351,9 @@ class Serialization(abc.ABC):
                 columns/rows/help_text->Column/Row label->
                 cell keys (value, description, language alias)->value
         """
+        if generate_schema:
+            return self.generate_json_schema()
+
         # Log warning if needed
         self.log_export_subset_warning_if_needed()
 
@@ -387,8 +426,14 @@ class Serialization(abc.ABC):
         # Export the spreadsheet to the dictionary (that can by JSON-ified)
         values = {x_start_key: {}}
         for idx_x in range(x_range):
+            if isinstance(x[idx_x], SkippedLabel):
+                # Skip labels that are intended to be skipped
+                continue
             y_values = {y_start_key: {}}
             for idx_y in range(y_range):
+                if isinstance(y[idx_y], SkippedLabel):
+                    # Skip labels that are intended to be skipped
+                    continue
                 # Select the correct cell
                 if by_row:
                     cell = self._get_cell_at(idx_x, idx_y)
@@ -401,23 +446,32 @@ class Serialization(abc.ABC):
                 # Replace the NaN value as required
                 if cell_value is None:
                     cell_value = nan_replacement
+                elif type(cell_value) == CellValueError:
+                    cell_value = error_replacement
                 # Receive values from cell (either integer or building text)
                 parsed_cell = cell.parse
+                # Following dict is the dict that is exported as a cell
                 pseudolang_and_val = {}
+                cell_description: str = cell.description
+                if cell_description is None:
+                    # Replace cell description to NaN replacement
+                    cell_description = nan_replacement
+                    # If the cell description should be equal to some language
+                    if descr_lang := use_language_for_description:  # noqa
+                        cell_description = parsed_cell[descr_lang]
+                # If it is an empty string, use None value instead
+                if cell_description == "":
+                    cell_description = nan_replacement
+                # Add description in all languages wanted (by aliases)
                 for i, language in enumerate(languages):
                     pseudolang_and_val[languages_used[i]] = \
                         parsed_cell[language]
                 # Append the value:
                 pseudolang_and_val['value'] = cell_value
-                pseudolang_and_val['description'] = cell.description
+                pseudolang_and_val['description'] = cell_description
                 y_values[y_start_key][y[idx_y]] = pseudolang_and_val
-                if y_helptext is not None:
-                    y_values[y_start_key][y[idx_y]]['help_text'] = \
-                        y_helptext[idx_y]
+
             values[x_start_key][x[idx_x]] = y_values
-            if x_helptext is not None:
-                values[x_start_key][x[idx_x]][
-                    'help_text'] = x_helptext[idx_x]
 
         # Create data parent
         data = {'data': values}
@@ -425,13 +479,42 @@ class Serialization(abc.ABC):
         # Add variables
         data['variables'] = self._get_variables().variables_dict
         # Add a row and column labels as arrays
-        if by_row:
-            data['row-labels'] = x
-            data['column-labels'] = y
-        else:
-            data['row-labels'] = y
-            data['column-labels'] = x
+        if not by_row:
+            x, y = y, x
 
+        # Add column/row metadata
+
+        # Add row description (and labels)
+        data['rows'] = []
+        for idx, x_label in enumerate(x):
+            if isinstance(x[idx_x], SkippedLabel):
+                # Skip labels that are intended to be skipped
+                continue
+            metadata: dict = {"name": x_label}
+            if x_helptext is not None:
+                # Add the help text (description) for the row
+                if (ht := x_helptext[idx]) is None or len(  # noqa
+                        x_helptext[idx]) == 0:
+                    # If it is not set or is empty string, use NaN value
+                    ht = nan_replacement
+                metadata['description'] = ht
+            data['rows'].append(metadata)
+
+        # Add column description (and labels)
+        data['columns'] = []
+        for idx, y_label in enumerate(y):
+            if isinstance(y[idx_y], SkippedLabel):
+                # Skip labels that are intended to be skipped
+                continue
+            metadata = {"name": y_label}
+            if y_helptext is not None:
+                # Add the help text (description) for the column
+                if (ht := y_helptext[idx]) is None or len(  # noqa
+                        y_helptext[idx]) == 0:
+                    # If it is not set or is empty string, use NaN value
+                    ht = nan_replacement
+                metadata['description'] = ht
+            data['columns'].append(metadata)
         # Create table parent
         table = {'table': data}
 
@@ -443,18 +526,25 @@ class Serialization(abc.ABC):
 
     def to_json(self,
                 languages: List[str] = None,
+                use_language_for_description: Optional[str] = None,
                 /, *,  # noqa E999
                 by_row: bool = True,
                 languages_pseudonyms: List[str] = None,
                 spaces_replacement: str = ' ',
                 skip_nan_cell: bool = False,
                 nan_replacement: object = None,
-                append_dict: dict = {}) -> str:
+                error_replacement: object = None,
+                append_dict: dict = {},
+                generate_schema: bool = False) -> str:
         """Dumps the exported dictionary to the JSON object.
 
         Args:
             languages (List[str]): List of languages that should be exported.
                 If it has value None, all the languages are exported.
+            use_language_for_description (Optional[str]): If set-up (using
+                the language name), description field is set to be either
+                the description value (if defined) or the value of this
+                language.
             by_row (bool): If True, rows are the first indices and columns
                 are the second in the order. If False it is vice-versa.
             languages_pseudonyms (List[str]): Rename languages to the strings
@@ -462,8 +552,10 @@ class Serialization(abc.ABC):
             spaces_replacement (str): All the spaces in the rows and columns
                 descriptions (labels) are replaced with this string.
             skip_nan_cell (bool): If True, None (NaN) values are skipped.
-            nan_replacement (object): Replacement for the None (NaN) value
+            nan_replacement (object): Replacement for the None (NaN) value.
+            error_replacement (object): Replacement for the error value.
             append_dict (dict): Append this dictionary to output.
+            generate_schema (bool): If true, returns the JSON schema.
 
         Returns:
             dict:
@@ -485,18 +577,205 @@ class Serialization(abc.ABC):
                     return super(_NumPyEncoder, self).default(obj)
         # Return correctly encoded JSON
         return json.dumps(
-            self.to_dictionary(languages,
+            self.to_dictionary(languages, use_language_for_description,
                                by_row=by_row,
                                languages_pseudonyms=languages_pseudonyms,
                                spaces_replacement=spaces_replacement,
                                skip_nan_cell=skip_nan_cell,
                                nan_replacement=nan_replacement,
-                               append_dict=append_dict),
+                               error_replacement=error_replacement,
+                               append_dict=append_dict,
+                               generate_schema=generate_schema),
             cls=_NumPyEncoder
         )
 
+    @staticmethod
+    def generate_json_schema() -> dict:
+        """Generate JSON schema.
+
+        Returns:
+            dict: JSON schema for the Portable Spreadsheet JSON table output.
+        """
+        false = False
+        schema = {
+            "$id": "http://portable-spreadsheet.com/spreadsheet.v3.schema.json",  # noqa
+            "$schema": "http://json-schema.org/draft-07/schema#",
+            "title": "Portable Spreadsheet JSON output schema",
+            "description": "JSON schema of the Portable Spreadsheet output",
+
+            "type": "object",
+            "required": ["table"],
+
+            "properties": {
+                "table": {
+                    "type": "object",
+                    "required": ["variables",
+                                 "rows",
+                                 "columns",
+                                 "data"],  # noqa
+                    "properties": {
+                        "rows": {
+                            "type": "array",
+                            "items": {
+                                "type": "object",
+                                "properties": {
+                                    "name": {
+                                        "anyOf": [
+                                            {"type": "string"},
+                                            {"type": "number"},
+                                            {"type": "null"}
+                                        ]
+                                    },
+                                    "description": {
+                                        "anyOf": [
+                                            {"type": "string"},
+                                            {"type": "number"},
+                                            {"type": "null"}
+                                        ]
+                                    },
+                                },
+                                "additionalProperties": false,
+                                "required": ['name']
+                            },
+                            "additionalProperties": false
+                        },
+                        "columns": {
+                            "type": "array",
+                            "items": {
+                                "type": "object",
+                                "properties": {
+                                    "name": {
+                                        "anyOf": [
+                                            {"type": "string"},
+                                            {"type": "number"},
+                                            {"type": "null"}
+                                        ]
+                                    },
+                                    "description": {
+                                        "anyOf": [
+                                            {"type": "string"},
+                                            {"type": "number"},
+                                            {"type": "null"}
+                                        ]
+                                    },
+                                },
+                                "additionalProperties": false,
+                                "required": ['name']
+                            },
+                            "additionalProperties": false
+                        },
+                        "variables": {
+                            "minProperties": 0,
+                            "type": "object",
+                            "propertyNames": {
+                                "type": "string"
+                            },
+                            "patternProperties": {
+                                "^.*$": {
+                                    "type": "object",
+                                    "properties": {
+                                        "value": {
+                                            "anyOf": [
+                                                {"type": "string"},
+                                                {"type": "number"},
+                                                {"type": "null"}
+                                            ]
+                                        },
+                                        "description": {
+                                            "anyOf": [
+                                                {"type": "string"},
+                                                {"type": "number"},
+                                                {"type": "null"}
+                                            ]
+                                        }
+                                    },
+                                    "required": ["value", "description"],
+                                    "additionalProperties": false
+                                }
+                            },
+                            "additionalProperties": false
+                        },
+                        "data": {
+                            "type": "object",
+                            "minProperties": 1,
+                            "maxProperties": 2,
+                            "additionalProperties": false,
+                            "patternProperties": {
+                                r"rows|columns": {
+                                    "type": "object",
+                                    "minProperties": 0,
+                                    "propertyNames": {
+                                        "type": "string"
+                                    },
+                                    "patternProperties": {
+                                        "^.*$": {
+                                            "type": "object",
+                                            "patternProperties": {
+                                                r"rows|columns": {
+                                                    "type": "object",
+                                                    "minProperties": 0,
+                                                    "propertyNames": {
+                                                        "type": "string"
+                                                    },
+                                                    "patternProperties": {
+                                                        "^.*$": {
+                                                            "type": "object",
+                                                            "minProperties": 2,
+                                                            "propertyNames": {
+                                                                "type": "string" # noqa
+                                                            },
+                                                            "patternProperties": {  # noqa
+                                                                "value": {
+                                                                    "anyOf": [
+                                                                        {"type": "string"},  # noqa
+                                                                        {"type": "number"},  # noqa
+                                                                        {"type": "null"}  # noqa
+                                                                    ]
+                                                                },
+                                                                "description": {  # noqa
+                                                                    "anyOf": [
+                                                                        {"type": "string"},  # noqa
+                                                                        {"type": "number"},  # noqa
+                                                                        {"type": "null"}  # noqa
+                                                                    ]
+                                                                },
+                                                                "^.*$": {
+                                                                    "anyOf": [
+                                                                        {"type": "string"},  # noqa
+                                                                        {"type": "number"},  # noqa
+                                                                        {"type": "null"}  # noqa
+                                                                    ]
+                                                                },
+                                                            },
+                                                            "required": ['value', 'description'],  # noqa
+                                                            "additionalProperties": false  # noqa
+                                                        }
+                                                    }
+                                                }
+                                            },
+                                            "minProperties": 1,
+                                            "maxProperties": 2,
+                                            "additionalProperties": false
+                                        }
+                                    },
+                                    "additionalProperties": false
+                                }
+                            }
+                        }
+                    },
+                    "additionalProperties": false
+                }
+            },
+            "additionalProperties": {
+                "type": "object"
+            }
+        }
+
+        return schema
+
     def to_string_of_values(self) -> str:
         """Export values inside table to the Python array definition string.
+            (Mainly helpful for debugging purposes)
 
         Returns:
             str: Python list definition string.
@@ -521,7 +800,8 @@ class Serialization(abc.ABC):
                    top_right_corner_text: str = "Sheet",
                    skip_labels: bool = False,
                    na_rep: Optional[object] = None,
-                   spaces_replacement: str = ' ',) -> List[List[object]]:
+                   spaces_replacement: str = ' ',
+                   skipped_label_replacement: str = '') -> List[List[object]]:
         """Export values 2 dimensional Python array.
 
         Args:
@@ -534,6 +814,8 @@ class Serialization(abc.ABC):
                 equals to None).
             skip_labels (bool): If true, first row and column with labels is
                 skipped
+            skipped_label_replacement (str): Replacement for the SkippedLabel
+                instances.
 
         Returns:
             List[List[object]]: Python array.
@@ -550,17 +832,21 @@ class Serialization(abc.ABC):
                 row.append(top_right_corner_text)
                 # Insert labels of columns:
                 for col_i in range(self.shape[1]):
-                    col = self.cell_indices.columns_labels[
+                    col_lbl = self.cell_indices.columns_labels[
                         col_i + self.export_offset[1]
-                    ]
-                    value = col.replace(' ', spaces_replacement)
-                    row.append(value)
+                    ].replace(' ', spaces_replacement)
+                    if isinstance(col_lbl, SkippedLabel):
+                        col_lbl = skipped_label_replacement
+                    row.append(col_lbl)
             else:
                 if not skip_labels:
                     # Insert labels of rows
-                    row.append(self.cell_indices.rows_labels[
+                    row_lbl = self.cell_indices.rows_labels[
                                   row_idx + self.export_offset[0]
-                              ].replace(' ', spaces_replacement))
+                              ].replace(' ', spaces_replacement)
+                    if isinstance(row_lbl, SkippedLabel):
+                        row_lbl = skipped_label_replacement
+                    row.append(row_lbl)
                 for col_idx in range(self.shape[1]):
                     # Append actual values:
                     cell_at_position = self._get_cell_at(row_idx, col_idx)
@@ -588,6 +874,7 @@ class Serialization(abc.ABC):
 
                sep: str = ',',
                line_terminator: str = '\n',
+               skipped_label_replacement: str = ''
                ) -> str:
         """Export values to the string in the CSV logic
 
@@ -602,14 +889,19 @@ class Serialization(abc.ABC):
             na_rep (str): Replacement for the missing data.
             skip_labels (bool): If true, first row and column with labels is
                 skipped
+            skipped_label_replacement (str): Replacement for the SkippedLabel
+                instances.
 
         Returns:
             str: CSV of the values
         """
         sheet_as_array = self.to_2d_list(
-            top_right_corner_text=top_right_corner_text, na_rep=na_rep,
-            skip_labels=skip_labels, spaces_replacement=spaces_replacement,
-            language=language
+            top_right_corner_text=top_right_corner_text,
+            na_rep=na_rep,
+            skip_labels=skip_labels,
+            spaces_replacement=spaces_replacement,
+            language=language,
+            skipped_label_replacement=skipped_label_replacement
         )
         export = ""
         for row_idx in range(len(sheet_as_array)):
@@ -626,7 +918,8 @@ class Serialization(abc.ABC):
                     top_right_corner_text: str = "Sheet",
                     skip_labels: bool = False,
                     na_rep: Optional[object] = '',
-                    spaces_replacement: str = ' '
+                    spaces_replacement: str = ' ',
+                    skipped_label_replacement: str = ''
                     ):
         """Export values to the string in the Markdown (MD) file logic
 
@@ -639,14 +932,19 @@ class Serialization(abc.ABC):
             na_rep (str): Replacement for the missing data.
             skip_labels (bool): If true, first row and column with labels is
                 skipped
+            skipped_label_replacement (str): Replacement for the SkippedLabel
+                instances.
 
         Returns:
             str: Markdown (MD) compatible table of the values
         """
         sheet_as_array = self.to_2d_list(
-            top_right_corner_text=top_right_corner_text, na_rep=na_rep,
-            skip_labels=skip_labels, spaces_replacement=spaces_replacement,
-            language=language
+            top_right_corner_text=top_right_corner_text,
+            na_rep=na_rep,
+            skip_labels=skip_labels,
+            spaces_replacement=spaces_replacement,
+            language=language,
+            skipped_label_replacement=skipped_label_replacement
         )
         export = ""
         for row_idx in range(len(sheet_as_array)):
@@ -723,7 +1021,8 @@ class Serialization(abc.ABC):
                       top_right_corner_text: str = "Sheet",
                       na_rep: str = '',
                       language_for_description: str = None,
-                      skip_labels: bool = False) -> str:
+                      skip_labels: bool = False,
+                      skipped_label_replacement: str = '') -> str:
         """Export values to the string in the HTML table logic
 
         Args:
@@ -736,6 +1035,8 @@ class Serialization(abc.ABC):
                 language (if the property description is not set).
             skip_labels (bool): If true, first row and column with labels is
                 skipped
+            skipped_label_replacement (str): Replacement for the SkippedLabel
+                instances.
 
         Returns:
             str: HTML table definition
@@ -757,7 +1058,9 @@ class Serialization(abc.ABC):
                     export += "<th>"
                     col = self.cell_indices.columns_labels[
                         col_i + self.export_offset[1]
-                        ]
+                        ].replace(' ', spaces_replacement)
+                    if isinstance(col, SkippedLabel):
+                        col = skipped_label_replacement
                     if (help_text :=  # noqa 203
                             self.cell_indices.columns_help_text) is not None:
                         title_attr = ' title="{}"'.format(
@@ -766,7 +1069,7 @@ class Serialization(abc.ABC):
                     else:
                         title_attr = ""
                     export += f'<a href="javascript:;" {title_attr}>'
-                    export += col.replace(' ', spaces_replacement)
+                    export += col
                     export += "</a>"
                     export += "</th>"
             else:
@@ -781,9 +1084,12 @@ class Serialization(abc.ABC):
                         title_attr = ""
                     export += "<td>"
                     export += f'<a href="javascript:;" {title_attr}>'
-                    export += self.cell_indices.rows_labels[
+                    row_lbl = self.cell_indices.rows_labels[
                                   row_idx + self.export_offset[0]
                                   ].replace(' ', spaces_replacement)
+                    if isinstance(row_lbl, SkippedLabel):
+                        row_lbl = skipped_label_replacement
+                    export += row_lbl
                     export += "</a>"
                     export += "</td>"
 
@@ -819,9 +1125,9 @@ class Serialization(abc.ABC):
         Returns:
             List[str]: List of column labels
         """
-        return self.cell_indices.columns_labels[
-            self.export_offset[1]:(self.export_offset[1] + self.shape[1])
-        ]
+        return self.cell_indices.columns_labels_str[self.export_offset[1]:(
+                self.export_offset[1] + self.shape[1])
+               ]
 
     @property
     def index(self) -> List[str]:
@@ -830,6 +1136,6 @@ class Serialization(abc.ABC):
         Returns:
             List[str]: List of row labels.
         """
-        return self.cell_indices.rows_labels[
-            self.export_offset[0]:(self.export_offset[0] + self.shape[0])
-        ]
+        return self.cell_indices.rows_labels_str[self.export_offset[0]:(
+                self.export_offset[0] + self.shape[0])
+               ]
