@@ -1,6 +1,9 @@
 import copy
 from numbers import Number
-from typing import Dict, Set, Tuple
+from typing import Dict, Set, Tuple, TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from .cell import Cell
 
 from .grammars import GRAMMARS
 
@@ -44,7 +47,7 @@ class WordConstructor(object):
             self.words: T_word = {key: "" for key in self.languages}
 
     @staticmethod
-    def init_from_new_cell(cell, /) -> 'WordConstructor':  # noqa E999
+    def init_from_new_cell(cell: 'Cell', /) -> 'WordConstructor':  # noqa E999
         """Initialise the words when the new cell is created.
 
         Args:
@@ -61,8 +64,8 @@ class WordConstructor(object):
             return WordConstructor(cell_indices=cell.cell_indices)
 
     @staticmethod
-    def _binary_operation(first,
-                          second,
+    def _binary_operation(first: 'Cell',
+                          second: 'Cell',
                           operation: str) -> 'WordConstructor':
         """General binary operation.
 
@@ -87,7 +90,8 @@ class WordConstructor(object):
         return instance
 
     @staticmethod
-    def add(first, second, /) -> 'WordConstructor':  # noqa E225
+    def add(first: 'Cell',
+            second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Add binary operation (+).
 
         Args:
@@ -101,7 +105,8 @@ class WordConstructor(object):
         return WordConstructor._binary_operation(first, second, "add")
 
     @staticmethod
-    def subtract(first, second, /) -> 'WordConstructor':  # noqa E225
+    def subtract(first: 'Cell',
+                 second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Subtract binary operation (-).
 
         Args:
@@ -115,7 +120,8 @@ class WordConstructor(object):
         return WordConstructor._binary_operation(first, second, "subtract")
 
     @staticmethod
-    def multiply(first, second, /) -> 'WordConstructor':  # noqa E225
+    def multiply(first: 'Cell',
+                 second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Multiply binary operation (*).
 
         Args:
@@ -129,7 +135,8 @@ class WordConstructor(object):
         return WordConstructor._binary_operation(first, second, "multiply")
 
     @staticmethod
-    def divide(first, second, /) -> 'WordConstructor':  # noqa E225
+    def divide(first: 'Cell',
+               second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Divide binary operation (/).
 
         Args:
@@ -143,7 +150,8 @@ class WordConstructor(object):
         return WordConstructor._binary_operation(first, second, "divide")
 
     @staticmethod
-    def modulo(first, second, /) -> 'WordConstructor':  # noqa E225
+    def modulo(first: 'Cell',
+               second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Modulo binary operation (%).
 
         Args:
@@ -157,7 +165,8 @@ class WordConstructor(object):
         return WordConstructor._binary_operation(first, second, "modulo")
 
     @staticmethod
-    def power(first, second, /) -> 'WordConstructor':  # noqa E225
+    def power(first: 'Cell',
+              second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Power binary operation (**).
 
         Args:
@@ -171,8 +180,8 @@ class WordConstructor(object):
         return WordConstructor._binary_operation(first, second, "power")
 
     @staticmethod
-    def logicalConjunction(first,
-                           second, /) -> 'WordConstructor':  # noqa E225
+    def logicalConjunction(first: 'Cell',
+                           second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Logical conjunction operation (and).
 
         Args:
@@ -187,8 +196,8 @@ class WordConstructor(object):
                                                  "logical-conjunction")
 
     @staticmethod
-    def logicalDisjunction(first,
-                           second, /) -> 'WordConstructor':  # noqa E225
+    def logicalDisjunction(first: 'Cell',
+                           second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Logical disjunction operation (or).
 
         Args:
@@ -203,7 +212,8 @@ class WordConstructor(object):
                                                  "logical-disjunction")
 
     @staticmethod
-    def equalTo(first, second, /) -> 'WordConstructor':  # noqa E225
+    def equalTo(first: 'Cell',
+                second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Equal to binary operation (==).
 
         Args:
@@ -217,7 +227,8 @@ class WordConstructor(object):
         return WordConstructor._binary_operation(first, second, "equal-to")
 
     @staticmethod
-    def notEqualTo(first, second, /) -> 'WordConstructor':  # noqa E225
+    def notEqualTo(first: 'Cell',
+                   second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Not equal to binary operation (!=).
 
         Args:
@@ -231,8 +242,8 @@ class WordConstructor(object):
         return WordConstructor._binary_operation(first, second, "not-equal-to")
 
     @staticmethod
-    def greaterThan(first,
-                    second, /) -> 'WordConstructor':  # noqa E225
+    def greaterThan(first: 'Cell',
+                    second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Greater than binary operation (>).
 
         Args:
@@ -246,8 +257,8 @@ class WordConstructor(object):
         return WordConstructor._binary_operation(first, second, "greater-than")
 
     @staticmethod
-    def greaterThanOrEqualTo(first,
-                             second, /) -> 'WordConstructor':  # noqa E225
+    def greaterThanOrEqualTo(first: 'Cell',
+                             second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Greater than or equal to binary operation (>=).
 
         Args:
@@ -262,7 +273,8 @@ class WordConstructor(object):
                                                  "greater-than-or-equal-to")
 
     @staticmethod
-    def lessThan(first, second, /) -> 'WordConstructor':  # noqa E225
+    def lessThan(first: 'Cell',
+                 second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Less than binary operation (<).
 
         Args:
@@ -276,8 +288,8 @@ class WordConstructor(object):
         return WordConstructor._binary_operation(first, second, "less-than")
 
     @staticmethod
-    def lessThanOrEqualTo(first,
-                          second, /) -> 'WordConstructor':  # noqa E225
+    def lessThanOrEqualTo(first: 'Cell',
+                          second: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Less than or equal to binary operation (<=).
 
         Args:
@@ -292,8 +304,8 @@ class WordConstructor(object):
                                                  "less-than-or-equal-to")
 
     @staticmethod
-    def concatenate(first,
-                    second) -> 'WordConstructor':
+    def concatenate(first: 'Cell',
+                    second: 'Cell') -> 'WordConstructor':
         """Concatenate values as two strings and create a word.
 
         Args:
@@ -305,8 +317,11 @@ class WordConstructor(object):
                 and two operands.
         """
         # Inline function for appending context to inputs
-        def _generate_word_string_concatenate(in_cell) -> T_word:
+        def _generate_word_string_concatenate(
+                in_cell: 'Cell') -> T_word:
             """Generate word for each operand.
+            Args:
+                in_cell (Cell): Input cell.
 
             Returns:
                 T_word: Words for each language.
@@ -318,7 +333,7 @@ class WordConstructor(object):
                     return in_cell._constructing_words.words
                 elif in_cell.cell_type == CellType.value_only:
                     words: T_word = {}
-                    for language in instance.languages:
+                    for language in in_cell.word.languages:
                         if isinstance(in_cell.value, str):
                             pref = GRAMMARS[language]['operations'][
                                 'concatenate']['string-value']['prefix']
@@ -345,7 +360,7 @@ class WordConstructor(object):
         return instance
 
     @staticmethod
-    def _aggregation_parse_cell(cell,
+    def _aggregation_parse_cell(cell: 'Cell',
                                 start_idx: Tuple[int, int],
                                 end_idx: Tuple[int, int],
                                 language: str
@@ -440,7 +455,8 @@ class WordConstructor(object):
         return cell_start, cell_end
 
     @staticmethod
-    def aggregation(cell_start, cell_end,
+    def aggregation(cell_start: 'Cell',
+                    cell_end: 'Cell',
                     grammar_method: str) -> 'WordConstructor':
         """General aggregation function.
 
@@ -481,7 +497,7 @@ class WordConstructor(object):
         return instance
 
     @staticmethod
-    def parse(cell) -> T_word:
+    def parse(cell: 'Cell') -> T_word:
         """Parse the cell word. This function is called when the cell should
             be inserted to spreadsheet.
 
@@ -507,7 +523,7 @@ class WordConstructor(object):
             return words
 
     @staticmethod
-    def raw(cell, words: T_word, /) -> 'WordConstructor':  # noqa: E225
+    def raw(cell: 'Cell', words: T_word, /) -> 'WordConstructor':  # noqa: E225
         """Returns the raw statement string.
 
         Args:
@@ -523,7 +539,7 @@ class WordConstructor(object):
         return instance
 
     @staticmethod
-    def empty(cell, /) -> 'WordConstructor':  # noqa E225
+    def empty(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Returns the empty string.
 
         Args:
@@ -539,7 +555,7 @@ class WordConstructor(object):
         return instance
 
     @staticmethod
-    def reference(cell, /) -> 'WordConstructor':  # noqa E225
+    def reference(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Return the reference to the cell.
 
         Args:
@@ -564,7 +580,7 @@ class WordConstructor(object):
         return instance
 
     @staticmethod
-    def constant(cell, /) -> 'WordConstructor':  # noqa E225
+    def constant(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Return the value of the cell.
 
         Args:
@@ -575,13 +591,19 @@ class WordConstructor(object):
         """
         instance = WordConstructor(cell_indices=cell.cell_indices)
         for language in instance.languages:
-            prefix = GRAMMARS[language]['cells']['constant']['prefix']
-            suffix = GRAMMARS[language]['cells']['constant']['suffix']
-            instance.words[language] = prefix + str(cell.value) + suffix
+            if isinstance(cell.value, str):
+                pref = GRAMMARS[language]['cells']['constant-string']['prefix']
+                suff = GRAMMARS[language]['cells']['constant-string']['suffix']
+            if isinstance(cell.value, Number):
+                pref = GRAMMARS[language]['cells'][
+                    'constant-numeric']['prefix']
+                suff = GRAMMARS[language]['cells'][
+                    'constant-numeric']['suffix']
+            instance.words[language] = pref + str(cell.value) + suff
         return instance
 
     @staticmethod
-    def variable(cell, /) -> 'WordConstructor':  # noqa E225
+    def variable(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Return the cell as a variable.
 
         Args:
@@ -609,7 +631,7 @@ class WordConstructor(object):
 
     @staticmethod
     def _unary_operator(*,
-                        cell,
+                        cell: 'Cell',
                         prefix_path: Tuple[str],
                         suffix_path: Tuple[str]) -> 'WordConstructor':
         """Word creation logic for a general unary operator.
@@ -635,7 +657,7 @@ class WordConstructor(object):
         return instance
 
     @staticmethod
-    def brackets(cell, /) -> 'WordConstructor':  # noqa E225
+    def brackets(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Add brackets around the cell.
 
         Args:
@@ -651,7 +673,7 @@ class WordConstructor(object):
         )
 
     @staticmethod
-    def logarithm(cell, /) -> 'WordConstructor':  # noqa E225
+    def logarithm(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Add logarithm definition context around the cell.
 
         Args:
@@ -667,7 +689,7 @@ class WordConstructor(object):
         )
 
     @staticmethod
-    def exponential(cell, /) -> 'WordConstructor':  # noqa E225
+    def exponential(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Add exponential definition context around the cell.
 
         Args:
@@ -683,7 +705,7 @@ class WordConstructor(object):
         )
 
     @staticmethod
-    def ceil(cell, /) -> 'WordConstructor':  # noqa E225
+    def ceil(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Add ceiling function definition context around the cell.
 
         Args:
@@ -699,7 +721,7 @@ class WordConstructor(object):
         )
 
     @staticmethod
-    def floor(cell, /) -> 'WordConstructor':  # noqa E225
+    def floor(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Add floor function definition context around the cell.
 
         Args:
@@ -715,7 +737,7 @@ class WordConstructor(object):
         )
 
     @staticmethod
-    def round(cell, /) -> 'WordConstructor':  # noqa E225
+    def round(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Add rounding function definition context around the cell.
 
         Args:
@@ -731,7 +753,7 @@ class WordConstructor(object):
         )
 
     @staticmethod
-    def abs(cell, /) -> 'WordConstructor':  # noqa E225
+    def abs(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Add absolute value definition context around the cell.
 
         Args:
@@ -747,7 +769,7 @@ class WordConstructor(object):
         )
 
     @staticmethod
-    def sqrt(cell, /) -> 'WordConstructor':  # noqa E225
+    def sqrt(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Add square root value definition context around the cell.
 
         Args:
@@ -763,7 +785,7 @@ class WordConstructor(object):
         )
 
     @staticmethod
-    def signum(cell, /) -> 'WordConstructor':  # noqa E225
+    def signum(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Add signum value definition context around the cell.
 
         Args:
@@ -779,7 +801,7 @@ class WordConstructor(object):
         )
 
     @staticmethod
-    def logicalNegation(cell, /) -> 'WordConstructor':  # noqa E225
+    def logicalNegation(cell: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Add logical negation definition context around the cell.
 
         Args:
@@ -796,9 +818,9 @@ class WordConstructor(object):
         )
 
     @staticmethod
-    def conditional(condition,
-                    consequent,
-                    alternative, /) -> 'WordConstructor':  # noqa E225
+    def conditional(condition: 'Cell',
+                    consequent: 'Cell',
+                    alternative: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Construct the word for the conditional statement (if-then-else).
 
         Args:
@@ -861,9 +883,9 @@ class WordConstructor(object):
         return instance
 
     @staticmethod
-    def offset(reference,
-               row_skip,
-               column_skip, /) -> 'WordConstructor':  # noqa E225
+    def offset(reference: 'Cell',
+               row_skip: 'Cell',
+               column_skip: 'Cell', /) -> 'WordConstructor':  # noqa E225
         """Construct the word for the offset statement (skip n rows and m
             columns from the referential position).
 
