@@ -258,6 +258,13 @@ class TestCellBinaryOperations(unittest.TestCase):
                                real_operation_fn(self.u_operand_1.value,
                                                  self.u_operand_2.value))
 
+    def test_string_equal(self):
+        """Test if the strings are interpreted correctly in operations"""
+        u_operand_str = Cell(None, None, "abc", cell_indices=self.cell_indices)
+        comp = (self.a_operand_1 == u_operand_str)
+        self.assertEqual(comp.parse['python_numpy'], 'values[3,4]=="abc"')
+        self.assertEqual(comp.parse['excel'], '=F5="abc"')
+
     def test_add(self):
         """Test adding"""
         # Method test
@@ -670,6 +677,15 @@ class TestCellAggregationFunctionality(unittest.TestCase):
         self._check_aggregate_function(Cell.irr, npf.irr, "IRR(", ")",
                                        "npf.irr(", ")")
 
+    def test_match_negative_before_positive(self):
+        """Test of finding the position of the last negative number in the
+            series that is located just before the first non-negative number.
+        """
+        self._check_aggregate_function(Cell.match_negative_before_positive,
+                                       lambda x: (np.argmin(np.array(x) < 0)),
+                                       "MATCH(0,", ")",
+                                       "np.argmin(", "<0)")
+
 
 class TestCellUnaryFunctionality(unittest.TestCase):
     """Test unary operators (functions with just one parameter).
